require 'pry'
require 'json'
require 'restclient'
require 'nokogiri'
require 'mongo_mapper'
require 'sidekiq'
require 'csv'
MongoMapper.connection = Mongo::MongoClient.new("localhost", :pool_size => 25, :pool_timeout => 60)
MongoMapper.database = "instagram_scraper"
class SampledPopulationUser
  include MongoMapper::Document
  include Sidekiq::Worker
  key :screen_name, String
  key :user_id, Integer
  key :content, Hash
  key :row_data, Hash
  sidekiq_options :queue => :insta_miner
  def get_datapoints_spu(spu)
    default_profile_pic_urls = ["https://scontent-iad3-1.cdninstagram.com/t51.2885-19/11351741_1614162902193084_959581767_a.jpg", "https://scontent-iad3-1.cdninstagram.com/t51.2885-19/11906329_960233084022564_1448528159_a.jpg"]
    {
      follower_count: spu.content["data"]["counts"]["followed_by"],
      following_count: spu.content["data"]["counts"]["follows"],
      media_count: spu.content["data"]["counts"]["media"],
      bio_empty: ((spu.content["data"]["bio"].nil? || spu.content["data"]["bio"].empty?) ? 1 : 0),
      bio_length: ((spu.content["data"]["bio"].nil? || spu.content["data"]["bio"].empty?) ? 0 : spu.content["data"]["bio"].length),
      bio_word_count: ((spu.content["data"]["bio"].nil? || spu.content["data"]["bio"].empty?) ? 0 : spu.content["data"]["bio"].split(" ").length),
      full_name_empty: ((spu.content["data"]["full_name"].nil? || spu.content["data"]["full_name"].empty?) ? 1 : 0),
      full_name_length: ((spu.content["data"]["full_name"].nil? || spu.content["data"]["full_name"].empty?) ? 0 : spu.content["data"]["full_name"].length),
      id: spu.content["data"]["id"],
      image_is_default_facebook: (default_profile_pic_urls.include?(spu.content["data"]["profile_picture"]) ? 1 : 0)
    }
  end
  
  def perform(dataset_name, id=(rand*3029051000).to_i)
    api_key = api_keys.shuffle.first
    data = nil
    begin
      data = RestClient.get("https://api.instagram.com/v1/users/#{id}/?access_token=#{api_key}")
    rescue RestClient::BadRequest => e
      data = e.response
    end
    result = JSON.parse(data)
    spu = SampledPopulationUser.first_or_create(user_id: id)
    spu.content = result
    if result["data"] && result["data"]["username"]
      spu.screen_name = result["data"]["username"]
      spu.row_data = get_datapoints_spu(spu)
    end
    spu.dataset_name = dataset_name
    spu.save!
  end
  
  def api_keys
    #below you should have line separated access tokens for instagram.
    %w{}
  end
  
  def self.generate_screenshots
    `mkdir random_sample_all_instagram_screenshots`
    csv = File.open("random_sample_users_all_instagram.csv", "w")
    csv.write(SampledPopulationUser.fields(:screen_name).collect(&:screen_name).join("\n"))
    csv.close
    `python save_screenshots.py random_sample_users_all_instagram.csv random_sample_all_instagram_screenshots`
  end
  
  def self.run_test
    1.upto(10000) do |x|
      SampledPopulationUser.perform_async("train", (rand*500000000).to_i)
      SampledPopulationUser.perform_async("test", (rand*500000000).to_i)
    end
  end
  
  def self.generate_paired_train_data
    csv = CSV.read("scores.csv")[1..-1].collect{|x| [x[0],x[1].to_i]}
    rows = Hash[SampledPopulationUser.where(dataset_name: "train", screen_name: csv.collect(&:first)).collect{|x| [x.screen_name, x.row_data]}]
    keys = rows.first.last.keys
    new_analysis_file = CSV.open("combined.csv", "w")
    new_analysis_file << ["score", keys].flatten
    bads = []
    csv.each do |pair|
      if rows[pair[0]]
        new_analysis_file << [pair[1], keys.collect{|k| rows[pair[0]][k]}].flatten
      else
        bads << pair
      end
    end
    new_analysis_file.close
    return keys
  end

  def self.generate_paired_test_data(keys)
    rows = Hash[SampledPopulationUser.where(dataset_name: "test").collect{|x| [x.screen_name, x.row_data]}].reject{|k,v| k.nil? || v.empty?}
    new_analysis_file = CSV.open("testing.csv", "w")
    new_analysis_file << [keys].flatten
    bads = []
    rows.values.each do |row|
      if !row.empty?
        new_analysis_file << [keys.collect{|k| row[k]}].flatten
      else
        bads << row
      end
    end
    new_analysis_file.close
  end
  
  def self.run_everything
    self.run_test
    #wait for all the data to be downloaded, generate your own scores.csv file with your assessment of particular accounts.
    keys = self.generate_paired_train_data
    self.generate_paired_test_data(keys)
  end
end